package com.ximba.videolib;

// Java
import java.io.*;
import java.net.*;
import java.util.*;
import org.apache.log4j.Logger;

// SWT 
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.browser.*;

/* Project Imports */
import com.ximba.common.*;

/**
 * <p> 
 * Select from alternate TMDB entries for a given video name.
 * </p>
 * @author Michael J. Hammel
 */

public class AlternatesDialog extends Dialog {

    private static final Logger log = Logger.getLogger("com.ximba.videolib.AlternatesDialog");

    public static final String S_TITLE = "Title Alternatives";

    /* Table Column Headers */
    public static final String[] tableHdrs  = {
        "Title",
        "Release Date",
        "ID",
    };

    /* Button Box */
    public static final int B_CANCEL = 0;
    public static final int B_ACCEPT = 1;
    public static final String[] bboxLabels  = {
        "Cancel",
        "Accept",
    };
    public Button[] bboxButtons  = new Button[bboxLabels.length];

    /*
     * GUI variables.
     */
    private Shell            shell = null;
    private Shell            parent = null;
    private boolean          isRunning = false;
    private Table            table = null;
    private Text             overview = null;

    private Video            video = null;

    private java.util.List<String> entries = null;

    /*
     * ------------------------------------------------------------------
     * Constructors.
     * ------------------------------------------------------------------
     */

    /**
     * Constructor requires a referenct to a parent shell and a reference
     * to the displayed note object that will be searched.
     */
    public AlternatesDialog(Shell parent, Video video) 
    {
        super(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
        this.parent = parent;
        this.video = video;
    }

    /*
     * ----------------------------------------------------------------
     * Inner classes
     * ----------------------------------------------------------------
     */

    /*
     * ------------------------------------------------------------------
     * Private Methods
     * ------------------------------------------------------------------
     */

    /** Position the dialog. */
    private void positionDialog()
    {
        Point pt = Display.getDefault().getCursorLocation();
        shell.setLocation(pt);
    }

    /*
     * ------------------------------------------------------------------
     * Public Methods
     * ------------------------------------------------------------------
     */

    /*
     * Scan for the movie @ tmdb.org.
     */
    public void scanAlternates()
    {
        if ( video == null )
        {
            log.error("Missing video for alternates scan.");
            return;
        }
        String movieName = video.getDisplayName();
        MovieDB mdb = new MovieDB();
        String query = mdb.buildMovieQuery( movieName );
        if ( query == null )
        {
            log.error("No query from buildMovieQuery.");
            return;
        }
        String jsonStr = mdb.makeRequest( query );
        if ( jsonStr != null ) 
            entries = mdb.parseQueryToList( jsonStr );
        else
        {
            log.error("No JSON from makeRequest.");
            return;
        }
        log.info("Found " + entries.size() + " alternates for " + movieName);
    }


    /** Update the video entry with the selected entry. */
    private void accept()
    {
        TableItem[] rows = table.getSelection();
        if ( rows.length == 0 )
            return;
        int tmdbID = -1;
        try { tmdbID = Integer.parseInt(rows[0].getText(2)); }
        catch(Exception e) {
            log.error("Failed to parse TMDB ID from selection: " + rows[0].getText(2));
        }
        video.setID(tmdbID);
        video.setReleaseDate( rows[0].getText(1) );
        video.setDisplayName( rows[0].getText(0) );
    }

    /** Close the dialog. */
    public void closeDialog()
    {
        isRunning = false;
    }

    /** Build and display the dialog. */
    public void open() 
    {
        isRunning = true;

        shell = new Shell(parent);
        shell.setText(S_TITLE);
        positionDialog();

        /* When the shell window closes, we know it's time to exit. */
        shell.addListener(SWT.Close, new Listener() {
            public void handleEvent (Event e) {
                isRunning = false;
            }
        });

        createContents(shell);
        populate();
        shell.pack();
        shell.open();
        shell.forceActive();

        /* Sit and spin until we're told to close. */
        while (isRunning) {
            if (!Display.getDefault().readAndDispatch ()) Display.getDefault().sleep ();
        }

        shell.close();
        shell.dispose();
    }

    /** Create the dialog content. */
    private void createContents(final Shell shell) 
    {
        Label label;
        GridData gd;
        GridLayout gl;

        /* The top level composite has 2 columns. */
        gl = new GridLayout();
        gl.marginWidth = 5;
        gl.marginHeight = 5;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 2;
        gl.makeColumnsEqualWidth = true;
        shell.setLayout(gl);

        table = new Table(shell, SWT.VIRTUAL | SWT.SINGLE | SWT.FULL_SELECTION | SWT.BORDER);
        table.setLinesVisible(true);
        table.setHeaderVisible(true);
        gd = new GridData(GridData.FILL_BOTH); 
        gd.horizontalSpan = 2;
        gd.widthHint = 500;
        gd.heightHint = 200;
        table.setLayoutData(gd);
        for (int i=0; i<tableHdrs.length; i++) {
            TableColumn column = new TableColumn (table, SWT.LEFT);
            column.setText (tableHdrs [i]);

            // Hide the ID column
            if ( i>1 )
                column.setWidth(0);
        }

        // Double clicks pull up the overview for the selected video.
        table.addListener (SWT.DefaultSelection, new Listener () {
            @Override
            public void handleEvent (Event e) {
                TableItem [] selection = table.getSelection();
                if ( (selection != null) && (selection.length == 1) )
                {
                    MovieDB mdb = new MovieDB();
                    String query = mdb.buildIDQuery( selection[0].getText(2) );
                    String jsonStr = mdb.makeRequest( query );
                    if ( jsonStr != null ) 
                        mdb.parseMovieData( jsonStr );
                    if ( mdb.getOverview() != null )
                        overview.setText( mdb.getOverview() );
                }
            }
        });

        overview = new Text(shell, SWT.WRAP | SWT.BORDER);
        gd = new GridData(GridData.FILL_BOTH); 
        gd.horizontalSpan = 2;
        gd.heightHint = 300;
        overview.setLayoutData(gd);

        /* -- Button box -- */
        Composite bc = new Composite(shell, SWT.BORDER);
        gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 2;
        bc.setLayoutData(gd);

        FormLayout fl = new FormLayout();
        fl.marginWidth = 0;
        fl.marginHeight = 0;
        bc.setLayout(fl);

        Button prevButton = null;
        for (int i=0; i < bboxLabels.length; i++)
        {
            final int idx = i;
            bboxButtons[i] = new Button(bc, SWT.NONE);
            bboxButtons[i].setText( bboxLabels[i] );
            bboxButtons[i].addSelectionListener(new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    switch(idx)
                    {
                        case B_CANCEL: 
                            closeDialog();   
                            break;
                        case B_ACCEPT: 
                            accept(); 
                            closeDialog();
                            break;
                    }
                }
            });
            FormData fd = new FormData();
            fd.bottom = new FormAttachment(100,0);
            if ( prevButton != null )
                fd.right = new FormAttachment(prevButton);
            else
                fd.right = new FormAttachment(100,0);
            bboxButtons[i].setLayoutData(fd);
            prevButton = bboxButtons[i];
        }
    }

    /** Populate the table with alternate movie data. */
    private void populate() 
    {
        if (table == null) 
            return;

        for (String entry: entries) 
        {
            String[] fields = entry.split(":");
            TableItem item = new TableItem (table, SWT.NONE);
            item.setText (0, fields[0]); // Title
            item.setText (1, fields[1]); // Release Date
            item.setText (2, fields[2]); // Video ID
        }
        for (int i=0; i<tableHdrs.length; i++) {
            table.getColumn (i).pack ();
        }
    }
}
