package com.ximba.videolib;

import java.io.*;
import java.util.*;
import java.net.*;
import org.apache.log4j.Logger;

/* Project Imports */
import com.ximba.common.*;

/**
 * <p> 
 * Manage local DB.
 * </p>
 * @author Michael J. Hammel
 */

public class LocalDB {

    /** lo4j logger */
    private static final Logger log = Logger.getLogger("com.ximba.videolib.LocalDB");

    // A place to store configuration settings.
    private static final Map<String, Video> db = new TreeMap<String, Video>();

    // Path to database.
    private String dbPath = null;

    /*
     * =======================================================
     * Constructor
     * =======================================================
     */

    public LocalDB() {}

    /*
     * =======================================================
     * Private methods 
     * =======================================================
     */

    /*
     * =======================================================
     * Public methods 
     * =======================================================
     */

    // Clear the database
    public void clear() { 
        log.info("db clear called.");
        db.clear(); 
    }

    // Get the database
    public Map<String, Video> get() { return db; }

    // Get an entry from the db.
    public Video getVideo(String displayName) {
        log.trace("Looking for video: " + displayName);
        log.trace("Number of videos: " + db.size());
        Iterator it = db.entrySet().iterator();
        while ( it.hasNext() )
        {
            Map.Entry pairs = (Map.Entry)it.next();
            log.trace("db key: " + (String)pairs.getKey() );
            if ( ((String)pairs.getKey()).equals( displayName ) )
            {
                log.info("Found video match for " + displayName);
                return (Video)pairs.getValue();
            }
        }
        return null;
    }

    // Update the key when the display name changes.
    public void updateKey( String oldKey, String newKey ) 
    {
        if ( (oldKey == null) | (newKey == null) )
            return;
        Video video = getVideo(oldKey);
        if ( video == null )
            return;
        Iterator it = db.entrySet().iterator();
        while ( it.hasNext() )
        {
            Map.Entry pairs = (Map.Entry)it.next();
            if ( ((String)pairs.getKey()).equals( oldKey ) )
            {
                it.remove();
                break;
            }
        }
        db.put( newKey, video );
    }

    // Add an individual video to the database
    public void add( Video video ) 
    {
        if ( video == null )
            return;
        if ( video.getDisplayName() == null )
            return;
        log.info("Adding video: " + video.getDisplayName());
        db.put( video.getDisplayName(), video );
    }
    
    /*
     * Database is saved into the directory specified.
     * It's has one row per video, with semicolon delimited fields.
     * All data is stored as text.
     */
    public void save( String dirName ) 
    {
        File dir = new File(dirName);
        if ( !dir.exists() )
        {
            log.error("No such directory.");
            return;
        }

        dbPath = dirName + File.separator + ".videolib.db";
        File file = new File(dbPath);
        file.delete();

        StringBuilder str = new StringBuilder("");
        Iterator it = db.entrySet().iterator();
        while ( it.hasNext() )
        {
            Map.Entry pairs = (Map.Entry)it.next();
            Video video = (Video)pairs.getValue();
            str.append( video.toString(dirName+File.separator) + "\n" );
        }
        PrintWriter out = null;
        try {
            out = new PrintWriter(dbPath);
            out.println( Video.hdrs() );
            out.println( str.toString() );
            out.close();
        }
        catch(Exception e){
            log.error("Failed to save db: " + e.getMessage(), e);
        }
        finally{
            if ( out != null )
                out.close();
        }
    }

    /*
     * Load the database.  This will clear the existing database.
     */
    public void load( String dirName ) 
    {
        if ( dirName == null )
            return;

        dbPath = dirName + File.separator + ".videolib.db";
        File file = new File(dbPath);
        if ( !file.exists() )
        {
            log.error("No such db: " + dbPath);
            return;
        }

        FileReader fr = null;
        BufferedReader in = null;
        db.clear();
        try {
            fr = new FileReader(dbPath);
            in = new BufferedReader(fr);
            String line = null;
            boolean hdr = true;
            while ( (line=in.readLine()) != null )
            {
                if ( hdr )
                {
                    hdr = false;
                    continue;
                }
                String[] fields = line.split(";");
                if ( fields.length < 6 )
                {
                    log.error("Not enough fields.  Skipping: " + line);
                    continue;
                }

                int id = -1;
                try { id = Integer.parseInt( fields[2] ); }
                catch(Exception e) {
                    log.error("Can't parse video ID: " + fields[3]);
                    continue;
                }

                Video video = new Video();
                file = new File( dirName + File.separator + fields[0] );
                video.setFile( file, dirName );
                video.setDisplayName( fields[1] );
                video.setID( id );
                video.setPoster( fields[3] );
                video.setReleaseDate( fields[4] );
                video.setOverview( fields[5] );

                db.put(video.getDisplayName(), video);
            }
            in.close();
            in = null;
            fr.close();
            fr = null;
        }
        catch (Exception e)
        {
            log.error("Can't read db (" + dbPath + ")", e);
        }
        finally
        {
            try {
                if ( in != null ) in.close();
                if ( fr != null ) fr.close();
            }
            catch(Exception e){}
        }
    }
}
