package com.ximba.videolib;

/* Java */
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.*;
import java.nio.*;
import java.nio.file.*;
import java.util.*;
import javax.imageio.ImageIO;
import org.apache.log4j.Logger;

// JSON-Simple
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

/**
 * <p> TheMovieDB API object that holds fields from a JSON return string. </p>
 * @author Michael J. Hammel
 * @since 1.0
 */
public class MovieDB implements Serializable {

    /* lo4j logger */
    private static final Logger log = Logger.getLogger("com.ximba.videolib.MovieDB");

    // ID of the matching movie.
    private String id            = null;
    private String release_date  = null;
    private String overview      = null;
    private String poster_path   = null;

    /*
     * ---------------------------------------------------------------
     * Constructors
     * ---------------------------------------------------------------
     */

    public MovieDB() { }

    /*
     * ---------------------------------------------------------------
     * Inner classes
     * ---------------------------------------------------------------
     */

    /*
     * ---------------------------------------------------------------
     * Private methods
     * ---------------------------------------------------------------
     */

    /*
     * ---------------------------------------------------------------
     * Public Methods
     * ---------------------------------------------------------------
     */

    public String makeRequest(String request)
    {
        BufferedReader in = null;
        try {
            URL url = new URL(request); 
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();           
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setInstanceFollowRedirects(false); 
            connection.setRequestMethod("GET"); 
            connection.setUseCaches (false);
            connection.disconnect();
        
            // Pull the results
            in = new BufferedReader( new InputStreamReader(connection.getInputStream()) );
            String decodedString;
            StringBuilder response = new StringBuilder("");
            while ((decodedString = in.readLine()) != null) {
                response.append(decodedString + "\n");
            }
            log.info("Response: " + response.toString());
            if ( response.toString().length() == 0 )
            {
                log.error("No data in response from Google.  Skipping.");
                return null;
            }
            return response.toString();
        }
        catch(Exception e)
        {
            log.error("Error making HTTP request: " + e.getMessage(), e);
            return null;
        }
        finally {
            try {
                if ( in != null )
                    in.close();
            }
            catch(Exception e){}
        }
    }

    public String requestImage(String request, String movieName, String wsize)
    {
        Cli cli = new Cli();
        if ( cli.get(Cli.S_MOVIEDIR) == null )
        {
            log.error("Movie dir is not set.  Skipping request for poster.");
            return null;
        }

        String posterPath = cli.get(Cli.S_MOVIEDIR) + File.separator + "posters" ;
        File dir = new File ( posterPath );
        dir.mkdirs();
        String fileName = posterPath + File.separator + 
                          movieName.replace(" ","_") + 
                          "-" + wsize +
                          ".png";

        log.info("Image filename: " + fileName);
        log.info("Request URL: " + request);
        try {
            URL url = new URL(request); 
            BufferedImage image = ImageIO.read(url);
            if ( ImageIO.write(image, "png", new File( fileName ) ) )
                return fileName.replace(cli.get(Cli.S_MOVIEDIR)+File.separator, "");
            else
            {
                log.error("Failed write - no image appropriate write available.");
                return null;
            }
        }
        catch(Exception e)
        {
            log.error("Error making HTTP request: " + e.getMessage(), e);
            return null;
        }
    }

    // Build a query based on a movie name.  We do a simple search, nothing more.
    // API: https://www.themoviedb.org/documentation/api
    // Example: http://stackoverflow.com/questions/14152276/themoviedb-json-api-with-jquery
    public String buildMovieQuery( String title )
    {
        Cli cli = new Cli();
        String apiKey = cli.get(Cli.S_APIKEY);
        if ( apiKey == null )
        {
            log.error("Missing API Key - can't query tmdb.org");
            return null;
        }
        try {
            StringBuilder url = new StringBuilder("http://api.themoviedb.org/3/search/movie?");
            url.append ( "api_key=" + apiKey );
            url.append ( "&query=" + URLEncoder.encode(title, "UTF-8") );
            return url.toString();
        }
        catch(Exception e) {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    // Build a query with a specific movie ID.  We do a simple search, nothing more.
    // API: https://www.themoviedb.org/documentation/api
    // Example: http://stackoverflow.com/questions/14152276/themoviedb-json-api-with-jquery
    public String buildIDQuery( String movieID )
    {
        Cli cli = new Cli();
        String apiKey = cli.get(Cli.S_APIKEY);
        if ( apiKey == null )
        {
            log.error("Missing API Key - can't query tmdb.org");
            return null;
        }
        StringBuilder url = new StringBuilder("http://api.themoviedb.org/3/movie/");
        url.append ( movieID + "?" );
        url.append ( "api_key=" + apiKey );
        return url.toString();
    }

    // Build image query.  This is hard coded but should use the "configuration" option to
    // find the correct base_url.
    // API: https://www.themoviedb.org/documentation/api
    // Example: http://stackoverflow.com/questions/14152276/themoviedb-json-api-with-jquery
    public String buildImageQuery( String movieID, String wsize )
    {
        if ( poster_path == null )
        {
            log.error("No poster path available - can't build image query.");
            return null;
        }
        Cli cli = new Cli();
        String apiKey = cli.get(Cli.S_APIKEY);
        if ( apiKey == null )
        {
            log.error("Missing API Key - can't query tmdb.org");
            return null;
        }

        StringBuilder url = new StringBuilder("http://image.tmdb.org/t/p");
        url.append ( "/" + wsize );
        url.append ( poster_path + "?" );
        url.append ( "api_key=" + apiKey );
        String newURL = url.toString();
        return url.toString();
    }

    // Parse JSON return value from TheMovieDB query.  Results are kept
    // internally to this class.
    public void parseQuery( String jsonStr )
    {
        try {
            //parses input json string to JSONObject
            JSONParser parser=new JSONParser();
            JSONObject jsonObject=(JSONObject)parser.parse(jsonStr);

            // Retrieve the array of results.
            JSONArray results = null;
            try { results = (JSONArray)jsonObject.get("results"); }
            catch(Exception e) {}
            if ( results == null )
            {
                log.error ("Missing results field in JSON data.");
                return;
            }
            log.error ("Missing results field in JSON data.");

            // Get first result. It's the most likely match.
            Iterator it = results.iterator();
            long id = -1;
            while( it.hasNext() )
            {
                JSONObject obj = (JSONObject)it.next();

                release_date = (String) obj.get( "release_date" );
                id = (long) obj.get("id");

                if ( id != -1 )
                    break;
            }

            // Save ID
            if ( id != -1 )
                this.id = ""+id;
            else
                log.error("Couldn't find ID");

        } catch (ParseException e) {
            log.info("Failed to parse JSON", e);
            e.printStackTrace();
        }
    }

    // Parse JSON return value from TheMovieDB movie ID lookup.  Results are kept
    // internally to this class.
    public void parseMovieData( String jsonStr )
    {
        try {
            //parses input json string to JSONObject
            JSONParser parser=new JSONParser();
            JSONObject jsonObject=(JSONObject)parser.parse(jsonStr);

            // Overview
            overview = (String)jsonObject.get("overview");
            log.info ("overview: " + overview);

            // Release date
            release_date = (String)jsonObject.get("release_date");
            log.info ("release_date: " + release_date);

            // Poster path
            poster_path = (String)jsonObject.get("poster_path");
            log.info ("poster_path: " + poster_path);

        } catch (ParseException e) {
            log.info("Failed to parse JSON", e);
            e.printStackTrace();
        }
    }

    // Parse JSON return value from TheMovieDB query into a list of 
    // colon seperated strings, and return that list.
    public java.util.List<String> parseQueryToList( String jsonStr )
    {
        java.util.List<String> list = new ArrayList<String>();

        try {
            //parses input json string to JSONObject
            JSONParser parser=new JSONParser();
            JSONObject jsonObject=(JSONObject)parser.parse(jsonStr);

            // Retrieve the array of results.
            JSONArray results = null;
            try { results = (JSONArray)jsonObject.get("results"); }
            catch(Exception e) {}
            if ( results == null )
            {
                log.error ("Missing results field in JSON data.");
                return list;
            }
            log.error ("Missing results field in JSON data.");

            // Get first result. It's the most likely match.
            Iterator it = results.iterator();
            long id = -1;
            while( it.hasNext() )
            {
                JSONObject obj = (JSONObject)it.next();

                String release = (String) obj.get( "release_date" );
                String tmdb_id = ""+obj.get("id");
                String title   = (String) obj.get("title");
                list.add( title+":"+release+":"+tmdb_id );
            }
        } catch (ParseException e) {
            log.info("Failed to parse JSON", e);
            e.printStackTrace();
        }
        return list;
    }

    public String getID() { return this.id; }
    public String getOverview() { return this.overview; }
    public String getReleaseDate() { return this.release_date; }
    public String getPosterPath() { return this.poster_path; }
}
