package com.ximba.videolib;

// Java
import java.io.*;
import java.net.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.util.*;
import java.util.concurrent.ConcurrentLinkedQueue;
import org.apache.log4j.Logger;

// SWT
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

// VideoLib
import com.ximba.common.*;

/**
 * <p> 
 * Front end to UI thread.
 * </p>
 * @author Michael J. Hammel
 * @since 1.0
 */

public class UI implements Runnable {

    /* lo4j logger */
    private static final Logger log = Logger.getLogger("com.ximba.videolib.UI");

    /*
     * --------------------------------------------
     * Thread Managment
     * --------------------------------------------
     */

    /* This thread. */
    private Thread thread = null;

    /* When true, the thread is running. */
    private boolean isRunning = false;

    /* When true, the thread has completed and is ready to change state to "not running". */
    private boolean runComplete = false;

    /* When shutting down, disable exception hanlding in run() method. */
    private boolean doShutdown = false;

    /*
     * --------------------------------------------
     * Fonts and Colors
     * --------------------------------------------
     */
    Font  boldFont       = null;
    Font  mediumFont     = null;
    Font  disclaimerFont = null;
    Color red            = null;

    /*
     * --------------------------------------------
     * Numerics
     * --------------------------------------------
     */

    /*
     * --------------------------------------------
     * Strings
     * --------------------------------------------
     */

    // The list of extensions supported by this application.
    public static final String[] extensions = {
        ".mp4",
        ".avi"
    };

    // tmdb disclaimer
    private static final String disclaimer = 
        "VideoLib uses the TMDb API but is not endorsed or certified by TMDb.";

    // The tmdb logo image
    private static final String logoFile = "images/tmdb-logo.png";

    /*
     * --------------------------------------------
     * UI Components
     * --------------------------------------------
     */
    Display display = null;
    Shell shell = null;
    org.eclipse.swt.widgets.List movieList = null;
    Text dirName = null;
    Button dirButton = null;
    Button scanButton = null;
    Button alternatesButton = null;
    Button buildButton = null;
    Button cancelButton = null;
    Text releaseDate = null;
    Text tmdbName = null;
    Text overview = null;
    Label posterLabel = null;
    Text apiText = null;
    Menu posterMenu = null;
    MenuItem[] posterItem = new MenuItem[9];
    AlternatesDialog adlg = null;

    boolean cancelBuild = false;

    /* Menus indices */
    private static final int M_FILE_QUIT            = 0;
    private static final int M_HELP_ABOUT           = 0;

    private static final int M_POSTER_92            = 0;
    private static final int M_POSTER_154           = 1;
    private static final int M_POSTER_185           = 2;
    private static final int M_POSTER_342           = 3;
    private static final int M_POSTER_500           = 4;
    private static final int M_POSTER_780           = 5;
    private static final int M_POSTER_ORIG          = 6;
    private static final int M_POSTER_SEP           = 7;
    private static final int M_POSTER_ALL           = 8;
    private static final int M_POSTER_DEFAULT       = 4;

    /*
     * Menus in Menu Bar
     */
    private static String[] FileMenuItems = {
        "&Quit"
    };
    private static String[] HelpMenuItems = {
        "&About",
    };
    private static String[] PosterMenuItems = {
      "92",
      "154",
      "185",
      "342",
      "500",
      "780",
      "original",
      "separator",
      "All",
    };

    /*
     * =======================================================
     * Constructor
     * =======================================================
     */

    void UI() { }

    /*
     * =======================================================
     * Inner classes
     * =======================================================
     */

    /*
     * =======================================================
     * Private UI methods
     * =======================================================
     */

    // Update the Movie List from the DB.
    private void updateFromDB()
    {
        movieList.setRedraw(false);
        movieList.removeAll();

        LocalDB localDB = new LocalDB();
        Iterator it = localDB.get().entrySet().iterator();
        while ( it.hasNext() )
        {
            Map.Entry pairs = (Map.Entry)it.next();
            Video video = (Video)pairs.getValue();
            movieList.add( video.getDisplayName() );
        }
        movieList.setRedraw(true);
    }

    /*
     * Update the details and poster based on a video entry.
     */
    private void updateDetails( Video video )
    {
        clearImage();
        releaseDate.setText( "" );
        overview.setText( "" );
        tmdbName.setText( "" );

        if ( video == null )
        {
            log.error("Video is null in updateDetails.");
            return;
        }

        if ( video.getReleaseDate() != null )
            releaseDate.setText( video.getReleaseDate() );
        if ( video.getOverview() != null )
        {
            try {
                overview.setText( URLDecoder.decode(video.getOverview(), "UTF-8") );
            }
            catch(Exception e) {
                log.warn("Failed to decode overview.");
                overview.setText( video.getOverview() );
            }
        }
        tmdbName.setText( video.getDisplayName() );
        if ( video.getPoster() != null )
        {
            String path = dirName.getText() + File.separator + video.getPoster();
            File file = new File(path);
            if ( file.exists() )
            {
                Image image = new Image(shell.getDisplay(), path);
                posterLabel.setImage(image);
            }
            else
                log.warn("Can't find poster: " + path);
        }
    }

    /*
     * Scan for a movie @ tmdb.org.
     */
    private long scanVideo( Video video )
    {
        if ( video == null )
        {
            log.error("Missing video for scanVideo.");
            return -1;
        }
        long startTime = new Date().getTime();
        String movieName = video.getDisplayName();
        MovieDB mdb = new MovieDB();
        String query = mdb.buildMovieQuery( movieName );
        if ( query == null )
        {
            log.error("No query from buildMovieQuery.");
            return -1;
        }
        String jsonStr = mdb.makeRequest( query );
        if ( jsonStr != null ) 
            mdb.parseQuery( jsonStr );
        else
        {
            log.error("No JSON from makeRequest.");
            return -1;
        }

        if ( mdb.getID() == null )
        {
            log.warn("No ID found. ");
            return -1;
        }
        log.info("Found ID: " + mdb.getID() );

        // Grab movie info
        query = mdb.buildIDQuery( mdb.getID() );
        jsonStr = mdb.makeRequest( query );
        if ( jsonStr != null ) 
            mdb.parseMovieData( jsonStr );
        if ( mdb.getReleaseDate() != null )
            releaseDate.setText( mdb.getReleaseDate() );
        if ( mdb.getOverview() != null )
            overview.setText( mdb.getOverview() );

        try { 
            video.setID( Integer.parseInt(mdb.getID()) ); 
        }
        catch (Exception e) {}
        video.setReleaseDate( mdb.getReleaseDate() );
        video.setOverview( mdb.getOverview() );

        // Request posters
        for(int i=0; i<M_POSTER_SEP; i++)
        {
            if ( posterItem[i].getSelection() )
            {
                String wsize;
                if ( i < M_POSTER_ORIG )
                    wsize = "w" + PosterMenuItems[i];
                else 
                    wsize = PosterMenuItems[i];
                String name = video.getDisplayName();
                query = mdb.buildImageQuery( mdb.getID(), wsize );
                if ( (name != null) && (query != null) )
                {
                    String filename = mdb.requestImage(query, name, wsize);
                    if ( (i == M_POSTER_DEFAULT) && (filename != null) )
                        video.setPoster( filename.replace(dirName.getText()+File.separator,"") );
                }
            }
        }

        updateDetails(video);

        long endTime = new Date().getTime();
        return endTime - startTime;
    }

    private void clearImage()
    {
        Image image = new Image(shell.getDisplay(), 800,800);
        GC gc = new GC(image);
        Color white = Display.getDefault().getSystemColor(SWT.COLOR_WHITE);
        gc.setForeground(white);
        gc.drawRectangle(0,0,800,800);
        Color black = Display.getDefault().getSystemColor(SWT.COLOR_BLACK);
        gc.setForeground(black);
        gc.drawText("No image available",0,0);
        gc.dispose();
        posterLabel.setImage(image);
    }

    /*
     * File menu handler
     */
    private void fileMenuHandler( int idx ) 
    {
        switch(idx)
        {
            case M_FILE_QUIT:   
                Cli cli = new Cli();
                cli.disable();
                break;
        }
    }

    /* Get a file listing in a directory */
    private java.util.List<File> getFileListingNoSort(File currentDir) throws FileNotFoundException {
        java.util.List<File> result = new ArrayList<>();
        File[] filesAndDirs = currentDir.listFiles();
        java.util.List<File> filesDirs = Arrays.asList(filesAndDirs);
        for(File file : filesDirs) {
            String name = file.getName();
            for(int i=0; i<extensions.length; i++)
            {
                if ( name.endsWith(extensions[i]) )
                    result.add(file); //Only add video files
            }
            if (! file.isFile()) {
                //must be a directory
                //recursive call!
                java.util.List<File> deeperList = getFileListingNoSort(file);
                result.addAll(deeperList);
            }
        }
        return result;
    }

    /** Choose a directory */
    private String selectDir()
    {
        DirectoryDialog dlg = new DirectoryDialog(shell, SWT.OPEN);
        dlg.setMessage("Choose a directory");

        File file = new File(dirName.getText());
        if ( file.exists() )
            dlg.setFilterPath( dirName.getText());
        else
            dlg.setFilterPath( System.getenv("user.home") );

        String filename = dlg.open();
        if ( filename == null )
            return null;
        return filename;
    }

    /* Build a list of movies */
    private void buildVideoList(String cwd) 
    {
        File currentDir = new File( cwd );
        java.util.List<File> results = null;
        try { results = getFileListingNoSort(currentDir); }
        catch(Exception e) {
            log.error("Failed to get video list.", e);
            return;
        }
        Collections.sort(results);
        Iterator it = results.iterator();
        while( it.hasNext() )
        {
            File file = (File)it.next();
            if ( file.isDirectory() )
                continue;
            Video video = new Video();
            video.setFile( file, dirName.getText()+File.separator );
            LocalDB localDB = new LocalDB();
            localDB.add( video );
            movieList.add( video.getDisplayName() );
        }

        LocalDB localDB = new LocalDB();
        it = localDB.get().entrySet().iterator();
        while ( it.hasNext() )
        {
            Map.Entry pairs = (Map.Entry)it.next();
            log.info("Key: *" + pairs.getKey() + "*");
        }
    }

    /* List of movies */
    private void buildListFrame() 
    {
        GridData gd = new GridData(GridData.FILL_BOTH);
        Composite composite = new Composite(shell, SWT.NONE);
        composite.setLayout(new GridLayout(1, false));
        composite.setLayoutData(gd);

        // API Key
        gd = new GridData(GridData.FILL_HORIZONTAL);
        Group group = new Group(composite, SWT.NONE);
        group.setText("TheMovieDB API Key");
        group.setFont(boldFont);
        group.setLayoutData(gd);

        GridLayout gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 2;
        gl.makeColumnsEqualWidth = false;
        group.setLayout(gl);

        gd = new GridData(GridData.FILL_HORIZONTAL);
        apiText = new Text(group, SWT.BORDER);
        apiText.setLayoutData(gd);

        Button button = new Button(group, SWT.PUSH);
        button.setText("Save");
        button.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                Cli cli = new Cli();
                cli.load();
                cli.set(Cli.S_APIKEY, apiText.getText());
                cli.save();
            }
            public void widgetDefaultSelected(SelectionEvent ev) {}
        });

        // Movie List
        gd = new GridData(GridData.FILL_BOTH);
        gd.grabExcessVerticalSpace = true;
        group = new Group(composite, SWT.NONE);
        group.setText("Movie List");
        group.setFont(boldFont);
        group.setLayoutData(gd);

        gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 2;
        gl.makeColumnsEqualWidth = false;
        group.setLayout(gl);

        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.widthHint = 200;
        dirName = new Text(group, SWT.BORDER);
        dirName.setLayoutData(gd);

        dirButton = new Button(group, SWT.PUSH);
        dirButton.setText("Select");
        dirButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                String dir = selectDir();
                if ( dir == null )
                    return;
                log.info("Dirname = " + dir);
                File file = new File( dir );
                if ( !file.exists() || !file.isDirectory() )
                {
                    log.error(dir + " does not exist.");
                    return;
                }
                dirName.setText(dir);
                Cli cli = new Cli();
                cli.set(Cli.S_MOVIEDIR, dir);
                cli.save();

                // Now build a lists of videos 
                log.info("Building file list for " + dirName.getText() );
                buildVideoList( dirName.getText() );
            }
            public void widgetDefaultSelected(SelectionEvent ev) {
            }
        });

        buildButton = new Button(group, SWT.PUSH);
        buildButton.setText("Build DB");
        buildButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                File dir = new File(dirName.getText());
                if ( !dir.exists() )
                    return;

                buildButton.setEnabled(false);
                cancelButton.setEnabled(true);

                // Run list of files and scan for each.
                String[] entries = movieList.getItems();
                if ( (entries == null) || (entries.length == 0) )
                    return;
                LocalDB localDB = new LocalDB();
                for(int i=0; i<entries.length; i++)
                {
                    // Only 30 requests per 10 seconds allowed.
                    // We slow things down a bit.
                    Video video = localDB.getVideo(entries[i]);
                    if ( video == null )
                    {
                        log.error("Can't find video in db.  Skipping " + entries[i]);
                        continue;
                    }
                    long runTime = scanVideo(video);
                    if ( runTime > 0 )
                    {
                        runTime = 325 - runTime;
                        try { Thread.sleep(runTime); }
                        catch(Exception e){}
                    }
                    display.update();
                    if ( cancelBuild )
                        break;
                }
                buildButton.setEnabled(true);
                cancelButton.setEnabled(false);
                cancelBuild = false;
                updateFromDB();
                localDB.save(dirName.getText());
            }
            public void widgetDefaultSelected(SelectionEvent ev) {
            }
        });
        gd = new GridData(GridData.FILL_HORIZONTAL);
        buildButton.setLayoutData(gd);

        cancelButton = new Button(group, SWT.PUSH);
        cancelButton.setText("Cancel");
        cancelButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                cancelBuild = true;
            }
            public void widgetDefaultSelected(SelectionEvent ev) {
            }
        });
        gd = new GridData(GridData.FILL_HORIZONTAL);
        cancelButton.setLayoutData(gd);
        cancelButton.setEnabled(false);

        gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 2;
        gd.heightHint = 500;
        movieList = new org.eclipse.swt.widgets.List(group, SWT.V_SCROLL);
        movieList.setLayoutData(gd);
        movieList.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                String[] entries = movieList.getSelection();
                if ( (entries == null) || (entries.length == 0) )
                    return;

                // Fill in details window.
                LocalDB localDB = new LocalDB();
                Video video = localDB.getVideo(entries[0]);
                if ( video != null )
                {
                    updateDetails(video);
                    log.info("Selected video: " + video.getDisplayName());
                }
                else 
                    log.error("No matching video in db");
            }
            public void widgetDefaultSelected(SelectionEvent ev) {
            }
        });
    }

    /* Movie Details */
    private void buildDetailsFrame() 
    {
        GridData gd = new GridData(GridData.FILL_BOTH);
        Composite composite = new Composite(shell, SWT.NONE);
        composite.setLayout(new GridLayout(1, false));
        composite.setLayoutData(gd);

        GridLayout gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 1;
        gl.makeColumnsEqualWidth = false;
        composite.setLayoutData(gd);

        Group group = new Group(composite, SWT.NONE);
        group.setText("Details");
        group.setFont(boldFont);
        gd = new GridData(GridData.FILL_BOTH);
        group.setLayoutData(gd);

        gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 4;
        gl.numColumns = 2;
        gl.makeColumnsEqualWidth = false;
        group.setLayout(gl);

        gd = new GridData(GridData.FILL_HORIZONTAL);
        scanButton = new Button(group, SWT.PUSH);
        scanButton.setText("Rescan");
        scanButton.setLayoutData(gd);
        scanButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                String[] items = movieList.getSelection();
                if ( (items == null) || (items.length==0) )
                    return;
                LocalDB localDB = new LocalDB();
                Video video = localDB.getVideo(items[0]);
                if ( video==null )
                {
                    log.error("No such video in DB.  Skipping rescan.");
                    return;
                }
                String oldName = video.getDisplayName();
                String newName = tmdbName.getText();
                video.setDisplayName(newName);
                localDB.updateKey(oldName, newName);
                clearImage();
                scanVideo(video);
                updateFromDB();
                localDB.save(dirName.getText());
            }
            public void widgetDefaultSelected(SelectionEvent ev) {
            }
        });

        gd = new GridData(GridData.FILL_HORIZONTAL);
        alternatesButton = new Button(group, SWT.PUSH);
        alternatesButton.setText("Alternates");
        alternatesButton.setLayoutData(gd);
        alternatesButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent ev) {
                String[] items = movieList.getSelection();
                if ( (items == null) || (items.length==0) )
                    return;
                LocalDB localDB = new LocalDB();
                Video video = localDB.getVideo(items[0]);
                if ( video==null )
                {
                    log.error("No such video in DB.  Skipping alternates.");
                    return;
                }
                String oldName = video.getDisplayName();
                adlg = new AlternatesDialog(shell, video);
                adlg.scanAlternates();
                adlg.open();
                adlg = null;
                localDB.updateKey(oldName, video.getDisplayName());
                clearImage();
                scanVideo(video);
                updateFromDB();
                localDB.save(dirName.getText());
            }
            public void widgetDefaultSelected(SelectionEvent ev) {
            }
        });

        // Movie name used in search
        Label label = new Label(group, SWT.NONE);
        label.setText("TMDB Name");
        tmdbName = new Text(group, SWT.BORDER);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        tmdbName.setLayoutData(gd);

        // Release Date
        label = new Label(group, SWT.NONE);
        label.setText("Release Date");
        releaseDate = new Text(group, SWT.BORDER);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        releaseDate.setLayoutData(gd);

        label = new Label(group, SWT.NONE);
        label.setText("Overview");
        gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
        label.setLayoutData(gd);

        overview = new Text(group, SWT.BORDER|SWT.MULTI|SWT.WRAP|SWT.V_SCROLL);
        gd = new GridData(GridData.FILL_BOTH);
        gd.heightHint = 200;
        gd.grabExcessHorizontalSpace = true;
        overview.setLayoutData(gd);

        // Disclaimer and logo
        Composite logoComposite = new Composite(composite, SWT.BORDER);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        logoComposite.setLayoutData(gd);

        gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 1;
        logoComposite.setLayout(gl);

        label = new Label(logoComposite, SWT.NONE);
        Image tmdbLogo = new Image(shell.getDisplay(), logoFile);
        label.setImage(tmdbLogo);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        label.setLayoutData(gd);

        Text text = new Text(logoComposite, SWT.MULTI|SWT.WRAP|SWT.READ_ONLY|SWT.CENTER);
        text.setText(disclaimer);
        text.setFont(disclaimerFont);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.widthHint = 320;
        // gd.grabExcessVerticalSpace = true;
        text.setLayoutData(gd);
    }

    /* Movie Poster */
    private void buildPosterFrame() 
    {
        GridData gd = new GridData(GridData.FILL_BOTH);
        Composite composite = new Composite(shell, SWT.NONE);
        composite.setLayout(new GridLayout(1, false));
        composite.setLayoutData(gd);

        gd = new GridData(GridData.FILL_BOTH);
        gd.widthHint = 400;
        gd.grabExcessVerticalSpace = true;

        Group group = new Group(composite, SWT.NONE);
        group.setText("Poster");
        group.setFont(boldFont);
        group.setLayoutData(gd);

        GridLayout gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 1;
        gl.makeColumnsEqualWidth = false;
        group.setLayout(gl);

        // Poster art
        posterLabel = new Label(group, SWT.NONE);
        gd = new GridData(GridData.FILL_BOTH);
        gd.heightHint = 500;
        posterLabel.setLayoutData(gd);
        Image image = new Image(shell.getDisplay(), 800,800);
        GC gc = new GC(image);
        Color white = Display.getDefault().getSystemColor(SWT.COLOR_WHITE);
        gc.setForeground(white);
        gc.drawRectangle(0,0,800,800);
        gc.dispose();
        posterLabel.setImage(image);

    }

    /*
     * Initialize the display components
     */
    private void buildUI() 
    {
        display = new Display();
        shell = new Shell(display);

        shell.addListener(SWT.Close, new Listener() {
            public void handleEvent(Event event) {
                Cli cli = new Cli();
                cli.disable();
            }
        });

        boldFont = new Font(Display.getDefault(), "Tahoma", 10, SWT.BOLD);
        mediumFont = new Font(Display.getDefault(), "Tahoma", 9, SWT.ITALIC);
        disclaimerFont = new Font(Display.getDefault(), "Courier", 8, SWT.NONE);
        red = new Color(Display.getDefault(), 250, 150, 150);

        // Add a menu bar across the top
        Menu menuBar = new Menu (shell, SWT.BAR);
        shell.setMenuBar ( menuBar );

        /* Add File Menu */
        MenuItem menuBar_File= new MenuItem (menuBar, SWT.CASCADE);
        menuBar_File.setText ("&File");
        Menu fileMenu = new Menu(shell, SWT.DROP_DOWN);
        menuBar_File.setMenu( fileMenu );
        for (int i=0; i<FileMenuItems.length; i++)
        {
            final int idx = i;
            MenuItem item = new MenuItem (fileMenu, SWT.CASCADE);
            item.setText( FileMenuItems[i] );
            item.addListener (SWT.Selection, new Listener () {
                public void handleEvent (Event e) {
                    fileMenuHandler(idx);
                }
            });
        }

        /* Add Poster Menu */
        MenuItem menuBar_Poster= new MenuItem (menuBar, SWT.CASCADE);
        menuBar_Poster.setText ("&Poster");
        posterMenu = new Menu(shell, SWT.DROP_DOWN);
        menuBar_Poster.setMenu( posterMenu );
        for (int i=0; i<PosterMenuItems.length; i++)
        {
            final int idx = i;
            switch(i)
            {
                case M_POSTER_ALL:
                    posterItem[i] = new MenuItem (posterMenu, SWT.CHECK);
                    posterItem[i].setText( PosterMenuItems[i] );
                    posterItem[i].addListener (SWT.Selection, new Listener () {
                        public void handleEvent (Event e) {
                            if ( posterItem[M_POSTER_ALL].getSelection() )
                            {
                                for(int j=0; j<posterItem.length; j++)
                                {
                                    if ( j != M_POSTER_SEP )
                                        posterItem[j].setSelection(true);
                                }
                            }
                            else
                            {
                                for(int j=0; j<posterItem.length; j++)
                                {
                                    if ( j == M_POSTER_SEP )
                                        continue;
                                    if ( j != M_POSTER_DEFAULT )
                                        posterItem[j].setSelection(false);
                                    else
                                        posterItem[j].setSelection(true);
                                }
                            }
                        }
                    });
                    break;
                case M_POSTER_SEP:
                    posterItem[i] = new MenuItem (posterMenu, SWT.SEPARATOR);
                    break;
                default:
                    posterItem[i] = new MenuItem (posterMenu, SWT.CHECK);
                    posterItem[i].setText( PosterMenuItems[i] );
                    posterItem[i].addListener (SWT.Selection, new Listener () {
                        public void handleEvent (Event e) {
                            // force M_POSTER_DEFAULT to always be on.
                        }
                    });
                    break;
            }
        }
        posterItem[M_POSTER_DEFAULT].setSelection(true);

        /* Add Help Menu */
        MenuItem menuBar_Help= new MenuItem (menuBar, SWT.CASCADE);
        menuBar_Help.setText ("&Help");
        Menu helpMenu = new Menu(shell, SWT.DROP_DOWN);
        menuBar_Help.setMenu( helpMenu );
        for (int i=0; i<HelpMenuItems.length; i++)
        {
            final int idx = i;
            MenuItem item = new MenuItem (helpMenu, SWT.CASCADE);
            item.setText( HelpMenuItems[i] );
            item.addListener (SWT.Selection, new Listener () {
                public void handleEvent (Event e) {
                    // aboutMenuHandler(idx);
                }
            });
        }

        // Create a grid below the menu bar
        GridLayout gl = new GridLayout();
        gl.marginWidth = 0;
        gl.marginHeight = 0;
        gl.verticalSpacing = 0;
        gl.horizontalSpacing = 0;
        gl.numColumns = 3;
        gl.makeColumnsEqualWidth = false;
        shell.setLayout(gl);

        // The List Frame
        buildListFrame();

        // The Details Frame
        buildDetailsFrame();

        // The Poster Frame
        buildPosterFrame();

        shell.pack();
    }

    /*
     * =======================================================
     * Public Methods - Thread Managmenet
     * =======================================================
     */

    /** 
     * Setup specific setup items.
     */
    boolean setup() 
    {
        if ( !isRunning )
        {
            return true;
        }
        return false;
    }

    /** 
     * Start the thread.
     * @return True if the thread is started, false if it is already running.
     * @throws IOException if the thread cannot start.
     */
    synchronized boolean start() throws IOException
    {
        /*
         * Create and start the thread.
         */
        if ( !isRunning )
        {
            // Start UI thread
            this.thread = new Thread(this, "VLIBUI");
            thread.start();

            /* Wait for thread to start. */
            Calendar startWait = Calendar.getInstance();
            while ( !isRunning )
            {
                Calendar now = Calendar.getInstance();
                now.add(Calendar.SECOND, (-1*5));
                if ( now.after(startWait) )
                    break;
            }
            if ( !isRunning )
            {
                thread.interrupt();
                thread = null;
                runComplete = false;
                throw new IOException("Timed out waiting for server thread to start.");
            }
            return true;
        }
        else
            return false;
    }

    /** 
     * Stop the thread.  This always succeeds and can be used to make
     * sure the thread is ready to be started again.
     */
    synchronized void shutdown()
    {
        if ( adlg != null )
        {
            adlg.closeDialog();
            adlg = null;
        }
        doShutdown = true;
        if ( isRunning )
        {
            isRunning = false;
            thread.interrupt();

            /* Wait for the thread to stop. */
            Calendar startWait = Calendar.getInstance();
            while ( !runComplete )
            {
                Calendar now = Calendar.getInstance();
                now.add(Calendar.SECOND, (-1*5));
                if ( now.after(startWait) )
                    break;
            }
            log.info("UI shutdown complete.");
        }
        doShutdown = false;
    }

    /** 
     * Handle inbound messages.
     */
    public void run()
    {
        if ( isRunning )
            return;
        runComplete = false;

        // Build the UI 
        buildUI();

        // Set visible fields
        Cli cli = new Cli();
        if ( cli.get(Cli.S_APIKEY) != null )
            apiText.setText( cli.get(Cli.S_APIKEY) );

        // Display the db, if one exists.
        updateFromDB();
        if ( cli.get(Cli.S_MOVIEDIR) != null )
            dirName.setText( cli.get(Cli.S_MOVIEDIR) );

        /*
         *----------------------------------------------------------------
         * Spin, waiting on shutdown.
         *----------------------------------------------------------------
         */
        log.info("Starting UI Thread.");
        isRunning = true;
        shell.open();
        while ( isRunning && !shell.isDisposed() )
        {
            if ( !display.readAndDispatch() )
                display.sleep();
        }
        if ( !shell.isDisposed() )
            shell.dispose();
        display.dispose();
        runComplete = true;
        log.info("UI thread exiting.");
    }
}
